%undefine _ld_as_needed

# open it when we get testsuite from source tar
%bcond_with check
%bcond_with manual

%global sover 1.4.0
%global major 1 %%(echo %{sover} | cut -d. -f1)
%global minor 4 %%(echo %{sover} | cut -d. -f2)
%global openmpi openmpi
%global mpich mpich
%global blaslib flexiblas
%global OPENBLASLINK -lflexiblas
%global OPENBLASLIB /libflexiblas.so
%global METISLINK -lmetis
%global METISLIB %{_libdir}/libmetis.so
%global METISINC %{_includedir}/metis.h

Summary: Solution of large, sparse, nonsymmetric systems of linear equations
Name:    superlu_dist
Version: 8.1.2
Release: 2%{?dist}
License: BSD
URL:     http://crd-legacy.lbl.gov/~xiaoye/SuperLU/
Source0: https://github.com/xiaoyeli/superlu_dist/archive/v%version/%name-%version.tar.gz
Source1: %name-make.inc

Patch3000: %name-inst.patch
Patch3001: %name-fix_pkgconfig_creation.patch
Patch3003: %name-scotch_parmetis.patch
Patch3004: %name-only_short_tests.patch

BuildRequires: scotch-devel gcc-c++ dos2unix chrpath cmake3 %{blaslib}-devel suitesparse-devel metis-devel

%description
SuperLU is a general purpose library for the direct solution of large, sparse, nonsymmetric systems
of linear equations.  The library is written in C and is callable from either C or Fortran program.
It uses MPI, OpenMP and CUDA to support various forms of parallelism.  It supports both real and 
complex datatypes, both single and double precision, and 64-bit integer indexing.  The library
routines performs an LU decomposition with partial pivoting and triangular system solves through
forward and back substitution.  The LU factorization routines can handle non-square matrices but the 
triangular solves are performed only for square matrices.  The matrix columns may be preordered
(before factorization) either through library or user supplied routines.  This preordering for 
sparsity is completely separate from the factorization.  Working precision iterative refinement
subroutines are provided for improved backward stability.  Routines are also provided to equilibrate
the system, estimate the condition number, calculate the relative backward error, and estimate error
bounds for the refined solutions.

This version uses MPI and OpenMP.

%package openmpi
Summary:       Solution of large, sparse, nonsymmetric systems of linear equations - openmpi
BuildRequires: openmpi-devel
BuildRequires: ptscotch-openmpi-devel
Requires:      gcc-gfortran

%description openmpi
SuperLU is a general purpose library for the direct solution of large, sparse, nonsymmetric systems
of linear equations.  The library is written in C and is callable from either C or Fortran program.
It uses MPI, OpenMP and CUDA to support various forms of parallelism.  It supports both real and 
complex datatypes, both single and double precision, and 64-bit integer indexing.  The library
routines performs an LU decomposition with partial pivoting and triangular system solves through
forward and back substitution.  The LU factorization routines can handle non-square matrices but the 
triangular solves are performed only for square matrices.  The matrix columns may be preordered
(before factorization) either through library or user supplied routines.  This preordering for 
sparsity is completely separate from the factorization.  Working precision iterative refinement
subroutines are provided for improved backward stability.  Routines are also provided to equilibrate
the system, estimate the condition number, calculate the relative backward error, and estimate error
bounds for the refined solutions.

This version uses MPI and OpenMP.

This is the openmpi version.

%package openmpi-devel
Summary: Development files for %name-openmpi
Requires: openmpi-devel
Requires: %name-openmpi = %version-%release
Provides: %name-openmpi-static = %version-%release

%description openmpi-devel
Development files for %name-openmpi

%package doc
Summary: Documentation for %name
BuildArch: noarch

%description doc
Documentation for %name

%package mpich
Summary:       Solution of large, sparse, nonsymmetric systems of linear equations - mpich
BuildRequires: mpich-devel
BuildRequires: ptscotch-mpich-devel  >= 6.0.5
BuildRequires: ptscotch-mpich-devel-parmetis  >= 6.0.5
Requires:      gcc-gfortran

%description mpich
SuperLU is a general purpose library for the direct solution of large, sparse, nonsymmetric systems
of linear equations.  The library is written in C and is callable from either C or Fortran program.
It uses MPI, OpenMP and CUDA to support various forms of parallelism.  It supports both real and 
complex datatypes, both single and double precision, and 64-bit integer indexing.  The library
routines performs an LU decomposition with partial pivoting and triangular system solves through
forward and back substitution.  The LU factorization routines can handle non-square matrices but the 
triangular solves are performed only for square matrices.  The matrix columns may be preordered
(before factorization) either through library or user supplied routines.  This preordering for 
sparsity is completely separate from the factorization.  Working precision iterative refinement
subroutines are provided for improved backward stability.  Routines are also provided to equilibrate
the system, estimate the condition number, calculate the relative backward error, and estimate error
bounds for the refined solutions.

This version uses MPI and OpenMP.

This is the mpich version.

%package mpich-devel
Summary: Development files for %name-mpich
Requires: mpich-devel
Requires: ptscotch-mpich-devel ptscotch-mpich-devel-parmetis
Requires: %name-mpich = %version-%release
Provides: %name-mpich-static = %version-%release

%description mpich-devel
Development files for %name-mpich

%prep
%autosetup -n superlu_dist-%version -N

%if %{with manual}
cp %SOURCE1 make.inc
%patch3000 -p1 -b .orig
%endif

dos2unix CMakeLists.txt
%patch3001 -p1 -b .fix_pkgconfig_creation
%patch3004 -p1 -b .only_short_tests

%build
%if %{with manual}
export CFLAGS="%build_cflags" LDFLAGS="%build_ldflags" CXXFLAGS="%build_cxxflags"
for m in %mpich %openmpi; do
case $m in
openmpi) %_openmpi_load ;;
mpich) %_mpich_load ;;
esac
find -name \*.[oa] | xargs rm 2>/dev/null || true
make SuperLUroot=$(pwd) BLASLIB=%{OPENBLASLINK} INCLUDEDIR=$(pwd)/SRC V=1
mkdir -p tmp $m
pushd tmp
ar x ../SRC/libsuperlu_dist.a
mpicxx -shared -Wl,-soname=libsuperlu_dist.so.%major \
      -o ../$m/libsuperlu_dist.so.%sover *.o -fopenmp \
      -lptscotchparmetis -lscotchmetis -lscotch -lptscotch \
      -lptscotcherr -lptscotcherrexit \
      %{?with_optimized_blas:%OPENBLASLINK} \
      %{?__global_ldflags}
popd
case $m in
openmpi)
  cp -a EXAMPLE/pddrive .
  %_openmpi_unload ;;
mpich)
  make -C EXAMPLE clean
  %_mpich_unload ;;
esac
done
%endif

%{_openmpi_load}
mkdir -p build/openmpi
export CC=$MPI_BIN/mpicc
export CXX=$MPI_BIN/mpic++
export CXXFLAGS="%optflags -I$MPI_INCLUDE"
export LDFLAGS="%build_ldflags -L$MPI_LIB -lptscotch -lptscotcherr -lptscotcherrexit"
%cmake3 -B build/openmpi -DCMAKE_BUILD_TYPE:STRING=Release \
 -DBUILD_STATIC_LIBS:BOOL=FALSE \
 -DCMAKE_Fortran_COMPILER:FILEPATH=$MPI_BIN/mpifort \
 -DMPIEXEC_EXECUTABLE:FILEPATH=$MPI_BIN/mpiexec \
 -DTPL_ENABLE_COLAMD=ON -DTPL_COLAMD_INCLUDE_DIRS:PATH=%{_includedir}/suitesparse -DTPL_COLAMD_LIBRARIES:STRING=%{_libdir}/libcolamd.so \
 -DMPI_C_LINK_FLAGS:STRING="-L$MPI_LIB -lptscotch -lptscotcherr -lptscotcherrexit -L%{_libdir} %{METISLINK} -lscotch -lcolamd" \
 -DTPL_BLAS_LIBRARIES:BOOL=ON -DTPL_BLAS_LIBRARIES:FILEPATH=%{_libdir}%{OPENBLASLIB} -DTPL_ENABLE_LAPACKLIB:BOOL=OFF -DTPL_LAPACK_LIBRARIES:BOOL=OFF \
 -DMPI_C_HEADER_DIR:PATH="$MPI_INCLUDE -I%{METISINC}" \
 -DMPI_CXX_LINK_FLAGS:STRING="-L$MPI_LIB -lptscotch -lptscotcherr -lptscotcherrexit -L%{_libdir} %{METISLINK} -lscotch -fopenmp" \
 -DTPL_PARMETIS_INCLUDE_DIRS:PATH=$MPI_INCLUDE \
 -DTPL_PARMETIS_LIBRARIES:STRING="$MPI_LIB/libptscotchparmetis.so;%{METISLIB}" \
 -DXSDK_INDEX_SIZE=32 \
 -DTPL_ENABLE_PARMETISLIB:BOOL=OFF \
 -Denable_double:BOOL=ON -Denable_complex16:BOOL=ON \
 -Denable_examples:BOOL=ON -Denable_tests:BOOL=ON -DBUILD_TESTING:BOOL=ON \
 -DCMAKE_INSTALL_PREFIX:PATH=%{_prefix} -DCMAKE_INSTALL_BINDIR:PATH=$MPI_BIN -DCMAKE_INSTALL_INCLUDEDIR:PATH=$MPI_INCLUDE/%{name} \
 -DCMAKE_INSTALL_LIBDIR:PATH=$MPI_LIB -DTPL_ENABLE_INTERNAL_BLASLIB:BOOL=ON -DCMAKE_SKIP_INSTALL_RPATH:BOOL=ON

%make_build V=1 -C build/openmpi
%{_openmpi_unload}

%{_mpich_load}
mkdir -p build/mpich
export CC=$MPI_BIN/mpicc
export CXX=$MPI_BIN/mpic++
export CFLAGS="%optflags -DPRNTlevel=0 -DDEBUGlevel=0"
export CXXFLAGS="%optflags -I$MPI_INCLUDE"
export LDFLAGS="%build_ldflags -L$MPI_LIB -lptscotch -lptscotcherr -lptscotcherrexit"
%cmake3 -B build/mpich -DCMAKE_BUILD_TYPE:STRING=Release \
 -DBUILD_STATIC_LIBS:BOOL=FALSE \
 -DCMAKE_Fortran_COMPILER:FILEPATH=$MPI_BIN/mpifort \
 -DMPIEXEC_EXECUTABLE:FILEPATH=$MPI_BIN/mpiexec \
 -DTPL_ENABLE_COLAMD=ON -DTPL_COLAMD_INCLUDE_DIRS:PATH=%{_includedir}/suitesparse -DTPL_COLAMD_LIBRARIES:STRING=%{_libdir}/libcolamd.so \
 -DMPI_C_LINK_FLAGS:STRING="-L$MPI_LIB -lptscotch -lptscotcherr -lptscotcherrexit -L%{_libdir} %{METISLINK} -lscotch -lcolamd" \
 -DTPL_BLAS_LIBRARIES:BOOL=ON -DTPL_BLAS_LIBRARIES:FILEPATH=%{_libdir}%{OPENBLASLIB} -DTPL_ENABLE_LAPACKLIB:BOOL=OFF -DTPL_LAPACK_LIBRARIES:BOOL=OFF \
 -DMPI_C_HEADER_DIR:PATH="$MPI_INCLUDE -I%{METISINC}" \
 -DMPI_CXX_LINK_FLAGS:STRING="-L$MPI_LIB -lptscotch -lptscotcherr -lptscotcherrexit -L%{_libdir} %{METISLINK} -lscotch" \
 -DTPL_PARMETIS_INCLUDE_DIRS:PATH=$MPI_INCLUDE \
 -DTPL_PARMETIS_LIBRARIES:STRING="$MPI_LIB/libptscotchparmetis.so;%{METISLIB}" \
 -DXSDK_INDEX_SIZE=32 \
 -DTPL_ENABLE_PARMETISLIB:BOOL=OFF \
 -Denable_double:BOOL=ON -Denable_complex16:BOOL=ON \
 -Denable_examples:BOOL=ON -Denable_tests:BOOL=ON -DBUILD_TESTING:BOOL=ON \
 -DCMAKE_INSTALL_PREFIX:PATH=%{_prefix} -DCMAKE_INSTALL_BINDIR:PATH=$MPI_BIN -DCMAKE_INSTALL_INCLUDEDIR:PATH=$MPI_INCLUDE/%{name} \
 -DCMAKE_INSTALL_LIBDIR:PATH=$MPI_LIB -DTPL_ENABLE_INTERNAL_BLASLIB:BOOL=ON -DCMAKE_SKIP_INSTALL_RPATH:BOOL=ON

%make_build -C build/mpich
%{_mpich_unload}

%install
%if %{with manual}
for m in %mpich %openmpi; do
case $m in
openmpi) %_openmpi_load ;;
mpich) %_mpich_load ;;
esac
mkdir -p %buildroot$MPI_LIB %buildroot$MPI_INCLUDE/superlu_dist
install -m644 SRC/*.h %buildroot$MPI_INCLUDE/superlu_dist

install -m 755 $m/libsuperlu_dist.so* %buildroot$MPI_LIB
pushd %buildroot$MPI_LIB
ln -s libsuperlu_dist.so.%sover libsuperlu_dist.so
ln -s libsuperlu_dist.so.%sover libsuperlu_dist.so.%major
ln -s libsuperlu_dist.so.%sover libsuperlu_dist.so.%major.%minor
popd
case $m in
openmpi) %_openmpi_unload ;;
mpich) %_mpich_unload ;;
esac
done
%endif

%{_openmpi_load}
%make_install -C build/openmpi
install -m644 SRC/*.h %buildroot$MPI_INCLUDE/superlu_dist/
rm -rf %buildroot$MPI_LIB/EXAMPLE
rm -rf %buildroot$MPI_LIB/superlu_dist/FORTRAN/CMakeFiles
chrpath -r $MPI_LIB %buildroot$MPI_LIB/libsuperlu_dist*.so*
%{_openmpi_unload}

%{_mpich_load}
%make_install -C build/mpich
install -m644 SRC/*.h %buildroot$MPI_INCLUDE/superlu_dist/

rm -rf %buildroot$MPI_LIB/EXAMPLE
rm -rf %buildroot$MPI_LIB/superlu_dist/FORTRAN/CMakeFiles
chrpath -r $MPI_LIB %buildroot$MPI_LIB/libsuperlu_dist*.so*
%{_mpich_unload}

%if %{with check}
%check
%if %{with manual}
%{?dts:source /opt/rh/devtoolset-7/enable}
pushd EXAMPLE
%_openmpi_load
export OMPI_MCA_rmaps_base_oversubscribe=1
mpirun -n 4 ../pddrive -r 2 -c 2 g20.rua
make clean
%endif

%{_openmpi_load}
%ifnarch %{ix86}
%ctest -- --test-dir build/openmpi -VV
%endif
%{_openmpi_unload}

%{_mpich_load}
%ctest -- --test-dir build/mpich -VV
%{_mpich_unload}
%endif

%files openmpi
%license License.txt
%_libdir/openmpi/lib/*.so.8
%_libdir/openmpi/lib/*.so.8.1.2

%files openmpi-devel
%_libdir/openmpi/lib/*.so
%_libdir/openmpi/lib/*.a
%_libdir/openmpi/lib/pkgconfig/*.pc
%_includedir/openmpi-%_arch/superlu_dist/

%files doc
%license License.txt
%doc DOC/ug.pdf EXAMPLE

%files mpich
%license License.txt
%_libdir/mpich/lib/*.so.8
%_libdir/mpich/lib/*.so.8.1.2

%files mpich-devel
%_libdir/mpich/lib/*.so
%_libdir/mpich/lib/*.a
%_libdir/mpich/lib/pkgconfig/*.pc
%_includedir/mpich-%_arch/superlu_dist/

%changelog
* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 8.1.2-2
- Rebuilt for OpenCloudOS Stream 23.05

* Wed Apr 26 2023 Shuo Wang <abushwang@tencent.com> - 8.1.2-1
- initial build
